package com.anzid.codegeneration.public_observers

import java.util.*

class ObserverObjectBuilder(model: ObserverGeneratorModel) {

    private val publicLiveDataName = model.fieldName.removePrefix("_")
    private val generatedDate = Date(System.currentTimeMillis()).toString()

    private val additionalContentTemplate = """
      
        /**
         * @see ${model.className}.${model.fieldName}
         */
        @Suppress("UNCHECKED_CAST")
        ${model.propertyModifiers}val ${model.className}.${publicLiveDataName} : ${model.type}
            get() {
                 val observerPrivate = javaClass.getDeclaredField("${model.fieldName}") 
                 observerPrivate.isAccessible = true 
                 val observer = observerPrivate.get(this) 
                 
                 return observer as ${model.type}
            }
        
    """.trimIndent()

    private val contentTemplate = """
@file:AnzidGenerationCode(
    value = ["Generated by AnzidProcessor", "com.anzid.codegeneration.public_observers.PublicObserversGenerator"],
    date = "Generated date: $generatedDate"
)

package ${model.pack}

import com.anzid.annotation.AnzidGenerationCode
import androidx.lifecycle.LiveData
import kotlinx.coroutines.flow.SharedFlow

$additionalContentTemplate
""".trimIndent()

    fun getDefaultContent() = contentTemplate
    fun getAdditionalContent() = additionalContentTemplate
}